import { ItemStack } from "../item/stack.js";
import { BlockStates } from "./states.js";
import { BlockTags } from "./tags.js";
import { BlockType } from "./type.js";
import { Wrapper } from "../wrapper.js";
import * as I from "../inspect.js";
import * as PP from "../pprint.js";
import * as MC from "@minecraft/server";
export class BlockPermutation extends Wrapper {
    #states;
    #tags;
    #type;
    constructor(...args) {
        switch (args.length) {
            case 1:
                if (typeof args[0] === "string")
                    super(MC.BlockPermutation.resolve(args[0]));
                else
                    super(args[0]);
                break;
            case 2:
                super(MC.BlockPermutation.resolve(args[0], args[1]));
                break;
            default:
                throw new Error(`Wrong number of arguments: ${args.length}`);
        }
    }
    get states() {
        if (!this.#states)
            this.#states = new BlockStates(this.raw);
        return this.#states;
    }
    get tags() {
        if (!this.#tags)
            this.#tags = new BlockTags(this.raw);
        return this.#tags;
    }
    get type() {
        if (!this.#type)
            this.#type = new BlockType(this.raw.type);
        return this.#type;
    }
    get typeId() {
        return this.type.id;
    }
    canBeDestroyedByLiquidSpread(liquidType) {
        return this.raw.canBeDestroyedByLiquidSpread(liquidType);
    }
    isLiquidBlocking(liquidType) {
        return this.raw.isLiquidBlocking(liquidType);
    }
    liquidSpreadCausesSpawn(liquidType) {
        return this.raw.liquidSpreadCausesSpawn(liquidType);
    }
    equals(other) {
        return this.raw.matches(other.raw.type.id, other.raw.getAllStates());
    }
    getItemStack(amount) {
        const rawStack = this.raw.getItemStack(amount);
        return rawStack ? new ItemStack(rawStack) : undefined;
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise) {
        const obj = {
            type: this.type
        };
        if (this.states.size > 0)
            obj.states = this.states;
        if (this.tags.size > 0)
            obj.tags = this.tags;
        return PP.spaceCat(stylise(PP.text("BlockPermutation"), I.TokenType.Class), inspect(obj));
    }
}
//# sourceMappingURL=permutation.js.map
