import * as I from "../inspect.js";
/** A read-only Map type that represents block permutation states. */
export class BlockStates {
    #perm;
    constructor(rawPerm) {
        this.#perm = rawPerm;
    }
    get size() {
        return Object.keys(this.#perm.getAllStates()).length;
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    *entries() {
        yield* Object.entries(this.#perm.getAllStates());
    }
    forEach(f, thisArg) {
        const boundF = f.bind(thisArg);
        for (const [key, value] of this) {
            boundF(value, key, this);
        }
    }
    get(key) {
        // The key is actually restricted to BlockStateSuperset from
        // @minecraft/vanilla-data, but making it depend on vanilla-data is
        // probably not the right thing to do.
        return this.#perm.getState(key);
    }
    has(key) {
        return this.get(key) !== undefined;
    }
    *keys() {
        yield* Object.keys(this.#perm.getAllStates());
    }
    *values() {
        yield* Object.values(this.#perm.getAllStates());
    }
    /// @internal
    [I.customInspectSymbol](inspect) {
        const obj = new Map(this);
        Object.defineProperty(obj, Symbol.toStringTag, { value: "BlockStates" });
        return inspect(obj);
    }
}
//# sourceMappingURL=states.js.map
