import { Wrapper } from "./wrapper.js";
/// @internal
export class Component extends Wrapper {
    get typeId() {
        return this.raw.typeId;
    }
    get isValid() {
        return this.raw.isValid;
    }
}
/** @internal We should be able to reduce the repetitive pattern in wrapper
 * classes having components like this:
 *
 * class Entity extends Wrapper<MC.Entity> {
 *     @component("minecraft:inventory", EntityInventory)
 *     readonly inventory: EntityInventory|undefined;
 * }
 *
 * ...but we can't, because MCBE currently uses an old version of QuickJS
 * that doesn't support ES2023.
 */
export function component(componentId, wrapperClass) {
    return (_target, context) => {
        if (context.static)
            throw new TypeError(`A field decorated with @component must not be static: ${String(context.name)}`);
        return function () {
            // We could cache wrapped components in a WeakMap but that's
            // probably slower than this.
            const raw = this.raw.getComponent(componentId);
            return raw ? new wrapperClass(raw) : undefined;
        };
    };
}
//# sourceMappingURL=component.js.map
