import { Block } from "./block.js";
import { Entity } from "./entity.js";
import { Wrapper } from "./wrapper.js";
import { map } from "./iterable.js";
import { WeatherType } from "@minecraft/server";
import * as I from "./inspect.js";
import * as PP from "./pprint.js";
import * as MC from "@minecraft/server";
export { WeatherType };
export class Dimension extends Wrapper {
    get heightRange() {
        return this.raw.heightRange;
    }
    get id() {
        return this.raw.id;
    }
    /** Get the block at the given location. This function never throws. */
    getBlock(location) {
        try {
            const raw = this.raw.getBlock(location);
            return raw ? new Block(raw) : undefined;
        }
        catch (e) {
            if (e instanceof MC.LocationInUnloadedChunkError ||
                e instanceof MC.LocationOutOfWorldBoundariesError)
                return undefined;
            else
                throw e;
        }
    }
    getEntities(opts) {
        return map(this.raw.getEntities(opts), raw => {
            return new Entity(raw);
        });
    }
    getWeather() {
        return this.raw.getWeather();
    }
    playSound(soundId, location, soundOptions) {
        this.raw.playSound(soundId, location, soundOptions);
    }
    setWeather(weatherType, duration) {
        this.raw.setWeather(weatherType, duration);
    }
    spawnEntity(identifier, location) {
        return new Entity(this.raw.spawnEntity(identifier, location));
    }
    spawnItem(itemStack, location) {
        return new Entity(this.raw.spawnItem(itemStack.raw, location));
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise) {
        const self = this;
        const obj = {
            id: this.id,
            heightRange: this.heightRange,
            get weather() {
                // This method is unavailable in read-only mode, although
                // it doesn't make sense.
                return self.getWeather();
            }
        };
        return PP.spaceCat(stylise(PP.text("Dimension"), I.TokenType.Class), inspect(obj));
    }
}
//# sourceMappingURL=dimension.js.map
