import { BlockPermutation } from "../block/permutation.js";
import { map } from "../iterable.js";
import { EntityComponent } from "./component.js";
import * as I from "../inspect.js";
import * as PP from "../pprint.js";
export class EntityBreathable extends EntityComponent {
    static typeId = "minecraft:breathable";
    set airSupply(value) {
        this.raw.airSupply = value;
    }
    get breatheBlocks() {
        return map(this.raw.getBreatheBlocks(), raw => {
            return new BlockPermutation(raw);
        });
    }
    get breathesAir() {
        return this.raw.breathesAir;
    }
    get breathesLava() {
        return this.raw.breathesLava;
    }
    get breathesSolids() {
        return this.raw.breathesSolids;
    }
    get breathesWater() {
        return this.raw.breathesWater;
    }
    get generatesBubbles() {
        return this.raw.generatesBubbles;
    }
    get inhaleTime() {
        return this.raw.inhaleTime;
    }
    get nonBreatheBlocks() {
        return map(this.raw.getNonBreatheBlocks(), raw => {
            return new BlockPermutation(raw);
        });
    }
    get suffocateTime() {
        return this.raw.suffocateTime;
    }
    get totalSupply() {
        return this.raw.totalSupply;
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise, opts) {
        const obj = {
            inhaleTime: this.inhaleTime,
            suffocateTime: this.suffocateTime,
            totalSupply: this.totalSupply,
        };
        if (this.breathesAir)
            obj.breathesAir = true;
        if (this.breathesLava)
            obj.breathesLava = true;
        if (this.breathesSolids)
            obj.breathesSolids = true;
        if (this.breathesWater)
            obj.breathesWater = true;
        if (this.generatesBubbles)
            obj.generatesBubbles = true;
        // Displaying the entire BlockPermutation for each block would be
        // too verbose. Do it when showHidden is enabled, otherwise only
        // show their block IDs.
        if (this.raw.getBreatheBlocks().length > 0) {
            if (opts.showHidden)
                obj.breatheBlocks = Array.from(this.breatheBlocks);
            else
                obj.breatheBlocks = Array.from(this.breatheBlocks).map(perm => perm.typeId);
        }
        if (this.raw.getNonBreatheBlocks().length > 0) {
            if (opts.showHidden)
                obj.nonBreatheBlocks = Array.from(this.nonBreatheBlocks);
            else
                obj.nonBreatheBlocks = Array.from(this.nonBreatheBlocks).map(perm => perm.typeId);
        }
        return PP.spaceCat(stylise(PP.text("EntityBreathable"), I.TokenType.Class), inspect(obj));
    }
}
//# sourceMappingURL=breathable.js.map
