import { Entity } from "../entity.js";
import { Location } from "../location.js";
import { Wrapper } from "../wrapper.js";
import { fromIterable, map } from "../iterable.js";
import { EntityComponent } from "./component.js";
import * as I from "../inspect.js";
import * as PP from "../pprint.js";
export class EntityRideable extends EntityComponent {
    static typeId = "minecraft:rideable";
    get controllingSeat() {
        return this.raw.controllingSeat;
    }
    get crouchingSkipInteract() {
        return this.raw.crouchingSkipInteract;
    }
    get familyTypes() {
        return fromIterable(this.raw.getFamilyTypes());
    }
    get interactText() {
        return this.raw.interactText;
    }
    get pullInEntities() {
        return this.raw.pullInEntities;
    }
    get riderCanInteract() {
        return this.raw.riderCanInteract;
    }
    get riders() {
        return map(this.raw.getRiders(), raw => {
            return new Entity(raw);
        });
    }
    get seatCount() {
        return this.raw.seatCount;
    }
    get seats() {
        return map(this.raw.getSeats(), raw => {
            return new Seat(raw);
        });
    }
    addRider(rider) {
        return this.raw.addRider(rider.raw);
    }
    ejectRider(rider) {
        return this.raw.ejectRider(rider.raw);
    }
    ejectRiders() {
        return this.raw.ejectRiders();
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise, opts) {
        const obj = {
            controllingSeat: this.controllingSeat,
        };
        if (this.crouchingSkipInteract)
            obj.crouchingSkipInteract = true;
        if (this.interactText !== "")
            obj.interactText = this.interactText;
        if (this.pullInEntities)
            obj.pullInEntities = true;
        if (this.riderCanInteract)
            obj.riderCanInteract = true;
        // Displaying the entire Entity for each rider would be too
        // verbose. Do it when showHidden is enabled, otherwise only show
        // their entity IDs.
        if (this.raw.getRiders().length > 0) {
            if (opts.showHidden)
                obj.riders = Array.from(this.riders);
            else
                obj.riders = Array.from(this.riders).map(rider => rider.id);
        }
        if (this.raw.getSeats().length > 0)
            obj.seats = Array.from(this.seats);
        return PP.spaceCat(stylise(PP.text("EntityRideable"), I.TokenType.Class), inspect(obj));
    }
}
class Seat extends Wrapper {
    get lockRiderRotation() {
        return this.raw.lockRiderRotation;
    }
    get maxRiderCount() {
        return this.raw.maxRiderCount;
    }
    get minRiderCount() {
        return this.raw.minRiderCount;
    }
    get position() {
        return new Location(this.raw.position);
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise, _opts) {
        const obj = {
            lockRiderRotation: this.lockRiderRotation,
            maxRiderCount: this.maxRiderCount,
            minRiderCount: this.minRiderCount,
            position: this.position,
        };
        return PP.spaceCat(stylise(PP.text("Seat"), I.TokenType.Class), inspect(obj));
    }
}
//# sourceMappingURL=rideable.js.map
