export class EntityTags {
    #entity;
    /// Package private.
    constructor(rawEntity) {
        this.#entity = rawEntity;
    }
    get size() {
        return this.#entity.getTags().length;
    }
    get [Symbol.toStringTag]() {
        return "EntityTags";
    }
    [Symbol.iterator]() {
        return this.values();
    }
    add(tag) {
        this.#entity.addTag(tag);
        return this;
    }
    clear() {
        for (const tag of this.#entity.getTags()) {
            this.#entity.removeTag(tag);
        }
    }
    delete(tag) {
        return this.#entity.removeTag(tag);
    }
    *entries() {
        for (const tag of this) {
            yield [tag, tag];
        }
    }
    forEach(f, thisArg) {
        const boundF = f.bind(thisArg);
        for (const tag of this) {
            boundF(tag, tag, this);
        }
    }
    has(tag) {
        return this.#entity.hasTag(tag);
    }
    keys() {
        return this.values();
    }
    values() {
        return this.#entity.getTags()[Symbol.iterator]();
    }
}
//# sourceMappingURL=tags.js.map
