import { EnchantmentType } from "@minecraft/server";
import * as I from "../inspect.js";
import * as PP from "../pprint.js";
import * as MC from "@minecraft/server";
export { EnchantmentType };
export class ItemEnchantments {
    /// @internal
    raw;
    static typeId = "minecraft:enchantable";
    static "type"(enchantmentId) {
        return MC.EnchantmentTypes.get(enchantmentId);
    }
    /// @internal
    constructor(raw) {
        this.raw = raw;
    }
    get size() {
        return this.raw ? this.raw.getEnchantments().length : 0;
    }
    get [Symbol.toStringTag]() {
        return "ItemEnchantments";
    }
    [Symbol.iterator]() {
        return this.entries();
    }
    canSet(type, level) {
        return this.raw
            ? this.raw.canAddEnchantment({
                type: typeof type === "string"
                    ? new EnchantmentType(type) : type,
                level
            })
            : false;
    }
    clear() {
        if (this.raw) {
            this.raw.removeAllEnchantments();
        }
    }
    delete(type) {
        if (this.has(type)) {
            this.raw.removeEnchantment(type);
            return true;
        }
        else {
            return false;
        }
    }
    *entries() {
        if (this.raw) {
            for (const ench of this.raw.getEnchantments()) {
                yield [ench.type, ench.level];
            }
        }
    }
    forEach(f, thisArg) {
        const boundF = f.bind(thisArg);
        for (const [type, level] of this) {
            boundF(level, type, this);
        }
    }
    "get"(type) {
        if (this.raw) {
            const ench = this.raw.getEnchantment(type);
            return ench ? ench.level : undefined;
        }
        else {
            return undefined;
        }
    }
    has(type) {
        if (this.raw) {
            return this.raw.hasEnchantment(type);
        }
        else {
            return false;
        }
    }
    *keys() {
        if (this.raw) {
            for (const ench of this.raw.getEnchantments()) {
                yield ench.type;
            }
        }
    }
    "set"(type, level) {
        if (this.raw) {
            this.raw.addEnchantment({
                type: typeof type === "string"
                    ? new EnchantmentType(type) : type,
                level
            });
            return this;
        }
        else {
            throw new TypeError("this item can have no enchantments");
        }
    }
    *values() {
        if (this.raw) {
            for (const ench of this.raw.getEnchantments()) {
                yield ench.level;
            }
        }
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise, opts) {
        /* [ItemEnchantments] {
         *     "silk_touch",
         *     "unbreaking" 1 (max: 3),
         *     "efficiency" 5 (max: 5)
         * }
         */
        const prefix = stylise(PP.text("ItemEnchantments"), I.TokenType.Class);
        const elems = [];
        for (const [type, level] of this) {
            const docs = [];
            docs.push(inspect(type.id));
            if (level > 1 || type.maxLevel > 1) {
                docs.push(inspect(level));
                docs.push(stylise(PP.parens(PP.spaceCat(PP.text("max:"), PP.number(type.maxLevel))), I.TokenType.Tag));
            }
            elems.push(PP.hsep(docs));
        }
        if (elems.length > 0)
            // If the entire object fits the line, print it in a single
            // line. Otherwise break lines for each enchantments.
            return PP.spaceCat(prefix, PP.group(PP.lineCat(PP.nest(opts.indentationWidth, PP.lineCat(PP.lbrace, PP.vsep(PP.punctuate(PP.comma, elems)))), PP.rbrace)));
        else
            return PP.spaceCat(prefix, PP.braces(PP.empty));
    }
}
I.overrideInspector(EnchantmentType, function (inspect, stylise) {
    return PP.hsep([
        stylise(PP.brackets(PP.text("EnchantmentType")), I.TokenType.Tag),
        stylise(PP.brackets(PP.text(`max: ${this.maxLevel}`)), I.TokenType.Tag),
        inspect(this.id)
    ]);
});
//# sourceMappingURL=enchantments.js.map
