import * as I from "../inspect.js";
/** A read-only set of item tags. It is mostly compatible with the Set
 * interface but no mutations can be performed.
 */
export class ItemTags {
    #stack;
    /// Package private.
    constructor(rawStack) {
        this.#stack = rawStack;
    }
    get size() {
        return this.#stack.getTags().length;
    }
    get [Symbol.toStringTag]() {
        return "ItemTags";
    }
    [Symbol.iterator]() {
        return this.values();
    }
    *entries() {
        for (const tag of this) {
            yield [tag, tag];
        }
    }
    forEach(f, thisArg) {
        const boundF = f.bind(thisArg);
        for (const tag of this) {
            boundF(tag, tag, this);
        }
    }
    has(tag) {
        return this.#stack.hasTag(tag);
    }
    keys() {
        return this.values();
    }
    values() {
        return this.#stack.getTags()[Symbol.iterator]();
    }
    /// @internal
    [I.customInspectSymbol](inspect) {
        const obj = new Set(this);
        Object.defineProperty(obj, Symbol.toStringTag, { value: "ItemTags" });
        return inspect(obj);
    }
}
//# sourceMappingURL=tags.js.map
