export function reversible(xs, reverse) {
    // We can't just create an object having reverse() and set its
    // prototype to xs, because calling next() on the original iterator
    // would involve a wrong `this` object.
    return new Proxy(xs, {
        get(_targ, key) {
            if (key === "reverse") {
                return reverse.bind(xs);
            }
            else {
                const prop = xs[key];
                return typeof prop === "function"
                    ? prop.bind(xs)
                    : prop;
            }
        }
    });
}
export function* fromIterable(xs) {
    for (const x of xs)
        yield x;
}
export function* map(xs, f) {
    for (const x of xs)
        yield f(x);
}
export function* concat(xs, ys) {
    yield* xs;
    yield* ys;
}
export function* intersperse(sep, xs) {
    let isFirst = true;
    for (const x of xs) {
        if (isFirst) {
            isFirst = false;
        }
        else {
            yield sep;
        }
        yield x;
    }
}
export function* snoc(xs, x) {
    yield* xs;
    yield x;
}
//# sourceMappingURL=iterable.js.map
