import * as I from "./inspect.js";
import * as PP from "./pprint.js";
export class Location {
    x;
    y;
    z;
    constructor(...args) {
        switch (args.length) {
            case 1:
                this.x = args[0].x;
                this.y = args[0].y;
                this.z = args[0].z;
                break;
            case 3:
                this.x = args[0];
                this.y = args[1];
                this.z = args[2];
                break;
            default:
                throw new Error("internal error");
        }
    }
    clone() {
        return new Location(this.x, this.y, this.z);
    }
    equals(other) {
        return this.x == other.x &&
            this.y == other.y &&
            this.z == other.z;
    }
    floor() {
        return new Location(Math.floor(this.x), Math.floor(this.y), Math.floor(this.z));
    }
    distance(other) {
        return Math.sqrt(Math.pow(this.x - other.x, 2) +
            Math.pow(this.y - other.y, 2) +
            Math.pow(this.z - other.z, 2));
    }
    isNear(other, epsilon) {
        return this.distance(other) <= epsilon;
    }
    offset(...args) {
        const clone = this.clone();
        if (args.length == 3) {
            clone.x += args[0];
            clone.y += args[1];
            clone.z += args[2];
        }
        else {
            clone.x += args[0];
            clone.y += args[0];
            clone.z += args[0];
        }
        return clone;
    }
    toString() {
        return `Location <${this.x}, ${this.y}, ${this.z}>`;
    }
    /// @internal
    [I.customInspectSymbol](inspect, stylise) {
        return PP.spaceCat(stylise(PP.text("Location"), I.TokenType.Class), PP.angles(PP.hsep(PP.punctuate(PP.comma, [inspect(this.x), inspect(this.y), inspect(this.z)]))));
    }
}
//# sourceMappingURL=location.js.map
