import { ConsoleBase, Severity } from "../console-base.js";
export class PlayerConsole extends ConsoleBase {
    #player;
    /** Package private */
    constructor(player) {
        super();
        this.#player = player;
    }
    logImpl(sev, ...args) {
        if (sev >= this.logLevel) {
            const msg = [
                this.severityToString(sev),
                this.timestamp(),
                ":",
                this.indent(),
                this.format(...args)
            ].join(""); // THINKME: Should we really join it?
            this.#player.sendMessage(msg);
        }
        // Also send it to the standard console.
        switch (sev) {
            case Severity.Debug:
                console.debug(...args);
                break;
            case Severity.Info:
                console.info(...args);
                break;
            case Severity.Log:
                console.log(...args);
                break;
            case Severity.Warning:
                console.warn(...args);
                break;
            case Severity.Error:
                console.error(...args);
                break;
        }
    }
}
//# sourceMappingURL=console.js.map
