export var Tag;
(function (Tag) {
    Tag[Tag["Fail"] = 0] = "Fail";
    Tag[Tag["Empty"] = 1] = "Empty";
    Tag[Tag["Text"] = 2] = "Text";
    Tag[Tag["Line"] = 3] = "Line";
    Tag[Tag["FlatAlt"] = 4] = "FlatAlt";
    Tag[Tag["Cat"] = 5] = "Cat";
    Tag[Tag["Nest"] = 6] = "Nest";
    Tag[Tag["Union"] = 7] = "Union";
    Tag[Tag["Column"] = 8] = "Column";
    Tag[Tag["Columns"] = 9] = "Columns";
    Tag[Tag["Nesting"] = 10] = "Nesting";
    Tag[Tag["Colour"] = 11] = "Colour";
    Tag[Tag["Obfuscate"] = 12] = "Obfuscate";
    Tag[Tag["Bold"] = 13] = "Bold";
    Tag[Tag["Italicise"] = 14] = "Italicise";
    Tag[Tag["RestoreFormat"] = 15] = "RestoreFormat";
})(Tag || (Tag = {}));
export var STag;
(function (STag) {
    STag[STag["SFail"] = 0] = "SFail";
    STag[STag["SEmpty"] = 1] = "SEmpty";
    STag[STag["SText"] = 2] = "SText";
    STag[STag["SLine"] = 3] = "SLine";
    STag[STag["SFormat"] = 4] = "SFormat";
})(STag || (STag = {}));
export const sFail = {
    tag: STag.SFail
};
export const sEmpty = {
    tag: STag.SEmpty
};
export function sText(text, succ) {
    return {
        tag: STag.SText,
        text,
        succ
    };
}
export function sLine(indent, content) {
    return {
        tag: STag.SLine,
        indent,
        content
    };
}
export function sFormat(codes, content) {
    return {
        tag: STag.SFormat,
        codes,
        content
    };
}
/** The empty document is, indeed, empty. Although `empty` has no
 * content, it does have a "height" of 1 and behaves exactly like
 * `text("")` (and is therefore not a unit of {@link lineCat}).
 */
export const empty = {
    tag: Tag.Empty
};
/** The document `text(s)` contains the literal string `s`. The string
 * shouldn't contain any newline (`"\n"`) characters. If the string
 * contains newline characters, the function {@link string} should be used.
 */
export function text(s) {
    return s.length == 0
        ? { tag: Tag.Empty }
        : { tag: Tag.Text, text: s };
}
/** The document `space` contains a single space, `" "`.
 */
export const space = {
    tag: Tag.Text,
    text: " "
};
/** A linebreak that will never be flattened; it is guaranteed to render as
 * a newline.
 */
export const hardline = {
    tag: Tag.Line
};
/** The `line` document advances to the next line and indents to the
 * current nesting level. Document `line` behaves like `text(" ")`
 * if the line break is undone by {@link group}.
 */
export const line = flatAlt(hardline, space);
/** The `linebreak` document advances to the next line and indents to
 * the current nesting level. Document `linebreak` behaves like
 * {@link empty} if the line break is undone by {@ group}.
 */
export const linebreak = flatAlt(hardline, empty);
/** The document `beside(x, y)` concatenates document `x` and document
 * `y`. It is an associative operation having {@link empty} as a left and right
 * unit.
 */
export function beside(x, y) {
    return {
        tag: Tag.Cat,
        fst: x,
        snd: y
    };
}
/** The document `nest(i, x)` renders document `x` with the current
 * indentation level increased by `i` (See also {@link hang}, {@link
 * align}, and {@link indent}).
 *
 * ``javascript
 * lineCat(nest(2, lineCat(text("hello"), text("world"))), text("!"))
 * ``
 *
 * outputs as:
 *
 * ``
 * hello
 *   world
 * !
 * ``
 */
export function nest(i, x) {
    return {
        tag: Tag.Nest,
        level: i,
        doc: x
    };
}
/** `column(f)` creates a document by applying the current column to the
 * given function `f`.
 */
export function column(f) {
    return {
        tag: Tag.Column,
        f: f
    };
}
/** `columns(f)` creates a document by applying the page width to the given
 * function `f`. If the document is being rendered without a finite page
 * width, a ``null`` value will be applied.
 */
export function columns(f) {
    return {
        tag: Tag.Columns,
        f: f
    };
}
/** `nesting(f)` creates a document by applying the current nesting level
 * (i.e. indentation level) to the given function `f`.
 */
export function nesting(f) {
    return {
        tag: Tag.Nesting,
        f: f
    };
}
/** A document that is normally rendered as the first argument, but when
 * flattened, is rendered as the second document.
 */
export function flatAlt(x, y) {
    return {
        tag: Tag.FlatAlt,
        fst: x,
        snd: y
    };
}
export const fail = {
    tag: Tag.Fail
};
export function union(x, y) {
    return {
        tag: Tag.Union,
        fst: x,
        snd: y
    };
}
//# sourceMappingURL=primitives.js.map
