/** Preferences in an arbitrary data structure that can be expressed in
 * Protocol Buffers proto3. They come in two flavours: per-player and
 * per-world preferences.
 */
import "./shims/text-decoder.js";
import "./shims/text-encoder.js";
import * as CicASCII from "./cic-ascii.js";
/** Package private */
export function dynamicPropertyId(type) {
    return `preferences.${type}`;
}
/** Package private */
export function decodeOrCreate(ty, prefs) {
    if (prefs === undefined) {
        return ty.create();
    }
    else {
        try {
            const bin = CicASCII.decode(prefs);
            return ty.fromBinary(bin);
        }
        catch (e) {
            console.error("Preference corrupted. Resetting: %o", e);
            return ty.create();
        }
    }
}
/** Package private */
export function encode(ty, prefs) {
    const bin = ty.toBinary(prefs);
    return CicASCII.encode(bin);
}
//# sourceMappingURL=preferences.js.map
