import { CustomEventSignal, GluedEventSignalWithoutOptions, GluedEventSignalWithOptions } from "../event.js";
import { Block, BlockPermutation, PlayerBreakBlockBeforeEvent } from "../block.js";
import { PistonActivateAfterEvent } from "../block/minecraft/piston.js";
import { Dimension } from "../dimension.js";
import { Entity, entityEventOptionsToRaw } from "../entity.js";
import { ItemStack, ItemUseBeforeEvent } from "../item.js";
import { ChatSendBeforeEvent, Player } from "../player.js";
import { Wrapper } from "../wrapper.js";
export class WorldAfterEvents extends Wrapper {
    playerBreakBlock;
    playerPlaceBlock;
    entityDie;
    itemUse;
    pistonActivate;
    worldLoad;
    /** An event that is fired when the world is fully loaded. */
    ready;
    playerSpawn;
    playerLeave;
    /// Package private
    constructor(rawEvents) {
        super(rawEvents);
        this.playerBreakBlock = new GluedEventSignalWithoutOptions(this.raw.playerBreakBlock, (rawEv) => {
            return {
                block: new Block(rawEv.block),
                dimension: new Dimension(rawEv.dimension),
                brokenBlockPermutation: new BlockPermutation(rawEv.brokenBlockPermutation),
                player: new Player(rawEv.player),
                ...(rawEv.itemStackAfterBreak
                    ? { itemStackAfterBreak: new ItemStack(rawEv.itemStackAfterBreak) }
                    : {}),
                ...(rawEv.itemStackBeforeBreak
                    ? { itemStackBeforeBreak: new ItemStack(rawEv.itemStackBeforeBreak) }
                    : {}),
            };
        });
        this.playerPlaceBlock = new GluedEventSignalWithoutOptions(this.raw.playerPlaceBlock, (rawEv) => {
            return {
                block: new Block(rawEv.block),
                dimension: new Dimension(rawEv.dimension),
                player: new Player(rawEv.player)
            };
        });
        this.entityDie = new GluedEventSignalWithOptions(this.raw.entityDie, (rawEv) => {
            return {
                damageSource: rawEv.damageSource,
                deadEntity: new Entity(rawEv.deadEntity)
            };
        }, entityEventOptionsToRaw);
        this.itemUse = new GluedEventSignalWithoutOptions(this.raw.itemUse, (rawEv) => {
            return {
                itemStack: new ItemStack(rawEv.itemStack),
                source: new Player(rawEv.source)
            };
        });
        this.pistonActivate = new GluedEventSignalWithoutOptions(this.raw.pistonActivate, (rawEv) => new PistonActivateAfterEvent(rawEv));
        this.worldLoad = this.raw.worldLoad;
        this.ready = new CustomEventSignal();
        this.playerSpawn = new CustomEventSignal();
        this.playerLeave = new CustomEventSignal();
    }
}
export class WorldBeforeEvents extends Wrapper {
    chatSend;
    itemUse;
    playerBreakBlock;
    playerLeave;
    /// Package private
    constructor(rawEvents) {
        super(rawEvents);
        this.chatSend = new GluedEventSignalWithoutOptions(this.raw.chatSend, (rawEv) => new ChatSendBeforeEvent(rawEv));
        this.itemUse = new GluedEventSignalWithoutOptions(this.raw.itemUse, (rawEv) => new ItemUseBeforeEvent(rawEv));
        this.playerBreakBlock = new GluedEventSignalWithoutOptions(this.raw.playerBreakBlock, (rawEv) => new PlayerBreakBlockBeforeEvent(rawEv));
        this.playerLeave = new GluedEventSignalWithoutOptions(this.raw.playerLeave, (rawEv) => {
            return {
                player: new Player(rawEv.player)
            };
        });
    }
}
//# sourceMappingURL=events.js.map
