var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _BlockPropertyRegistry_instances, _BlockPropertyRegistry_tags, _BlockPropertyRegistry_blocks, _BlockPropertyRegistry_cache, _BlockPropertyRegistry_getUncached;
import { LootTable, LootPool } from "./loot-table.js";
export var MiningWay;
(function (MiningWay) {
    /// Don't mine this block.
    MiningWay[MiningWay["LeaveAlone"] = 0] = "LeaveAlone";
    /// Mine the block in the normal way: the tool durability should be
    /// consumed.
    MiningWay[MiningWay["MineRegularly"] = 1] = "MineRegularly";
    /// Mine the block as a bonus: the tool durability should not be
    /// consumed.
    MiningWay[MiningWay["MineAsABonus"] = 2] = "MineAsABonus";
})(MiningWay || (MiningWay = {}));
/** A class describing the nature of a specific block type (as opposed to
 * block permutation). This is essentially a class of class but since
 * TypeScript doesn't support abstract static properties these classes are
 * implemented as singletons. See
 * https://github.com/microsoft/TypeScript/issues/34516
 */
export class BlockProperties {
    /** See if the block should be protected from getting mined.
     */
    isProtected(_perm, _player, _prefs) {
        return false;
    }
    /** See if the block breaks itself when it's not attached to an
     * adjacent block. Keep in mind that this isn't perfect, that is,
     * fragile blocks MUST NOT be quick-mined if their only supporting
     * blocks are also fragile.
     */
    isFragile(_perm) {
        return false;
    }
    /** Break a block. Most blocks turns into either `minecraft:air` or
     * `minecraft:water` depending on whether it is waterlogged, but there
     * are some special cases like ice.
     */
    "break"(block, _tool) {
        if (block.isWaterlogged)
            block.typeId = "minecraft:water";
        else
            block.typeId = "minecraft:air";
    }
    /** See if mining the block should consume durability of the tool.
     */
    consumesDurability(_perm, _tool) {
        return true;
    }
    /** See if two blocks should be considered equivalent. In most cases
     * this means `pa.equals(pb)`, which is the default implementation, but
     * some blocks (such as leaves) need to ignore certain block states
     * (e.g. `update_bit`).
     */
    isEquivalentTo(pa, pb) {
        return pa.equals(pb);
    }
    /** Get the amount of experience yielded by mining the block.
     */
    experience(_perm, _tool) {
        return 0;
    }
    /** Get the loot table for the block. The default implementation
     * returns a loot table dropping the block itself regardless of how
     * it's broken.
     */
    lootTable(perm) {
        const stack = perm.getItemStack(1);
        if (stack)
            return new LootTable()
                .always([
                new LootPool().entry(stack)
            ]);
        else
            // No corresponding items exist for this block. Drop nothing.
            return new LootTable();
    }
    /** Determine if or how `perm` should be quick-mined as a consequence
     * of initiating a quick mining at `origin`.
     */
    miningWay(origin, perm, _prefs) {
        return this.isEquivalentTo(origin, perm)
            ? MiningWay.MineRegularly
            : MiningWay.LeaveAlone;
    }
}
class DefaultBlockProperties extends BlockProperties {
    breakingSoundId(perm) {
        throw new Error(`The block ${perm.typeId} isn't meant to be quick-mined`);
    }
    isToolSuitable() {
        return false;
    }
}
const DEFAULT_BLOCK_PROPERTIES = new DefaultBlockProperties();
export class BlockPropertyRegistry {
    constructor() {
        _BlockPropertyRegistry_instances.add(this);
        _BlockPropertyRegistry_tags.set(this, void 0);
        _BlockPropertyRegistry_blocks.set(this, void 0);
        _BlockPropertyRegistry_cache.set(this, void 0);
        __classPrivateFieldSet(this, _BlockPropertyRegistry_tags, new Map(), "f");
        __classPrivateFieldSet(this, _BlockPropertyRegistry_blocks, new Map(), "f");
        __classPrivateFieldSet(this, _BlockPropertyRegistry_cache, new Map(), "f");
    }
    addTaggedProps(tag, propClass) {
        if (__classPrivateFieldGet(this, _BlockPropertyRegistry_tags, "f").has(tag))
            throw new Error(`Tagged properties redefined for tag ${tag}`);
        else
            __classPrivateFieldGet(this, _BlockPropertyRegistry_tags, "f").set(tag, new propClass());
        return this;
    }
    addBlockProps(blockId, propClass) {
        if (__classPrivateFieldGet(this, _BlockPropertyRegistry_blocks, "f").has(blockId))
            throw new Error(`Block properties redefined for block ID ${blockId}`);
        else
            __classPrivateFieldGet(this, _BlockPropertyRegistry_blocks, "f").set(blockId, new propClass());
        return this;
    }
    /** This method takes `BlockPermutation` as opposed to `BlockType` only
     * because the latter does not have tags. It's not that the returned
     * `BlockProperties` will retain the permutation.
     */
    "get"(block) {
        let props = __classPrivateFieldGet(this, _BlockPropertyRegistry_cache, "f").get(block.typeId);
        if (!props) {
            props = __classPrivateFieldGet(this, _BlockPropertyRegistry_instances, "m", _BlockPropertyRegistry_getUncached).call(this, block);
            __classPrivateFieldGet(this, _BlockPropertyRegistry_cache, "f").set(block.typeId, props);
        }
        return props;
    }
}
_BlockPropertyRegistry_tags = new WeakMap(), _BlockPropertyRegistry_blocks = new WeakMap(), _BlockPropertyRegistry_cache = new WeakMap(), _BlockPropertyRegistry_instances = new WeakSet(), _BlockPropertyRegistry_getUncached = function _BlockPropertyRegistry_getUncached(block) {
    // Block tags are always preferred over individual IDs.
    for (const tag of block.tags) {
        const props = __classPrivateFieldGet(this, _BlockPropertyRegistry_tags, "f").get(tag);
        if (props)
            return props;
    }
    const props = __classPrivateFieldGet(this, _BlockPropertyRegistry_blocks, "f").get(block.typeId);
    if (props)
        return props;
    // Having no specific block properties means that the block cannot
    // be quick-mined.
    return DEFAULT_BLOCK_PROPERTIES;
};
// @ts-ignore: Intentionally calling a private constructor.
export const blockProps = new BlockPropertyRegistry();
//# sourceMappingURL=block-properties.js.map
