var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _loots, _a, _loots_1, _b, _silkPoolFor, _nonSilkPoolFor, _lootsFor, _c;
import { ItemStack } from "../../../cicada-lib/dist/item/stack.js";
import { BlockProperties, blockProps } from "../../block-properties.js";
import { LootCondition, LootTable, LootPool } from "../../loot-table.js";
/// Base class for clay-like blocks.
class ClayLikeProperties extends BlockProperties {
    isToolSuitable(_perm, tool, prefs) {
        if (prefs.coverage.clayLike)
            return tool.tags.has("minecraft:is_shovel");
        else
            return false;
    }
}
/// Mixin for snow-like blocks.
function SnowLike(base) {
    class SnowLike extends base {
        breakingSoundId() {
            return "dig.snow";
        }
        isEquivalentTo(pa, pb) {
            // Ignore the difference in snow layers and snow blocks. Also
            // ignore the height of snow layers. But we must leave
            // snow-logged plants alone (see below).
            if (pa.typeId === "minecraft:snow_layer" || pa.typeId === "minecraft:snow")
                if (pb.typeId === "minecraft:snow_layer" || pb.typeId === "minecraft:snow")
                    return !pa.states.get("covered_bit") && !pb.states.get("covered_bit");
            return false;
        }
    }
    return SnowLike;
}
// Clay
blockProps.addBlockProps("minecraft:clay", (_a = class extends ClayLikeProperties {
        constructor() {
            super(...arguments);
            _loots.set(this, new LootTable()
                .when(LootCondition.matchTool().enchantment("silk_touch"), [new LootPool().entry(new ItemStack("minecraft:clay"))])
                .otherwise([new LootPool().entry(new ItemStack("minecraft:clay_ball", 4))]));
        }
        breakingSoundId() {
            return "dig.gravel";
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots, "f");
        }
    },
    _loots = new WeakMap(),
    _a));
// Mud
blockProps.addBlockProps("minecraft:mud", class extends ClayLikeProperties {
    breakingSoundId() {
        return "block.mud.break";
    }
});
// Snow block
blockProps.addBlockProps("minecraft:snow", (_b = class extends SnowLike(ClayLikeProperties) {
        constructor() {
            super(...arguments);
            _loots_1.set(this, new LootTable()
                .when(LootCondition.matchTool().enchantment("silk_touch"), [new LootPool().entry(new ItemStack("minecraft:snow"))])
                .otherwise([new LootPool().entry(new ItemStack("minecraft:snowball", 4))]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_1, "f");
        }
    },
    _loots_1 = new WeakMap(),
    _b));
// Snow layer
blockProps.addBlockProps("minecraft:snow_layer", (_c = class extends SnowLike(ClayLikeProperties) {
        constructor() {
            super(...arguments);
            _silkPoolFor.set(this, new Array(8).fill(0).map((_, height) => {
                if (height == 7)
                    return new LootPool().entry(new ItemStack("minecraft:snow"));
                else
                    return new LootPool().entry(new ItemStack("minecraft:snow_layer", height + 1));
            }));
            _nonSilkPoolFor.set(this, new Array(8).fill(0).map((_, height) => {
                switch (height) {
                    case 0:
                    case 1:
                    case 2:
                        return new LootPool().entry(new ItemStack("minecraft:snowball", 1));
                    case 3:
                    case 4:
                        return new LootPool().entry(new ItemStack("minecraft:snowball", 2));
                    case 5:
                    case 6:
                        return new LootPool().entry(new ItemStack("minecraft:snowball", 3));
                    case 7:
                        return new LootPool().entry(new ItemStack("minecraft:snowball", 4));
                    default:
                        throw new Error(`impossible case`);
                }
            }));
            _lootsFor.set(this, new Array(8).fill(0).map((_, height) => {
                return new LootTable()
                    .when(LootCondition.matchTool().enchantment("silk_touch"), [__classPrivateFieldGet(this, _silkPoolFor, "f")[height]])
                    .otherwise([__classPrivateFieldGet(this, _nonSilkPoolFor, "f")[height]]);
            }));
        }
        isToolSuitable(perm, tool, prefs) {
            // We must leave snow-logged plants alone because their plant
            // type is inaccessible from the scripting API atm. It seems to
            // be stored in NBT but there's no block components for
            // that. This means we cannot uncover plants without removing
            // them entirely.
            if (perm.states.get("covered_bit"))
                return false;
            else
                return super.isToolSuitable(perm, tool, prefs);
        }
        lootTable(perm) {
            const height = perm.states.get("height");
            const loots = __classPrivateFieldGet(this, _lootsFor, "f")[height];
            if (!loots)
                throw new Error(`Unknown height of snow layer: ${height}`);
            return loots;
        }
    },
    _silkPoolFor = new WeakMap(),
    _nonSilkPoolFor = new WeakMap(),
    _lootsFor = new WeakMap(),
    _c));
//# sourceMappingURL=clay.js.map
