var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _loots, _a, _loots_1, _b, _loots_2, _c, _loots_3, _d, _loots_4, _e, _loots_5, _f, _loots_6, _g, _loots_7, _h, _loots_8, _j, _loots_9, _k, _loots_10, _l;
import { ItemStack } from "../../../cicada-lib/dist/item/stack.js";
import { BlockProperties, MiningWay, blockProps } from "../../block-properties.js";
import { LootCondition, LootTable, LootPool } from "../../loot-table.js";
import { IgnoringState } from "../mixins.js";
const AZALEA_LEAVES_IDS = new Set([
    "minecraft:azalea_leaves",
    "minecraft:azalea_leaves_flowered",
]);
function lootOfLeaves(leafBlock, extraPools) {
    return new LootTable()
        .when(LootCondition.or([
        LootCondition.matchTool().typeId("minecraft:shears"),
        LootCondition.matchTool().enchantment("silk_touch")
    ]), [new LootPool().entry(leafBlock)]) // 100% drop
        .otherwise([new LootPool()
            .rolls(1, 2)
            .condition(LootCondition.randomChance(1 / 50, 1 / 45, 1 / 40, 1 / 30))
            .entry(new ItemStack("minecraft:stick")),
        ...extraPools
    ]);
}
/// Base class for all leaf blocks.
class LeavesProperties extends IgnoringState(BlockProperties, "update_bit") {
    breakingSoundId() {
        return "dig.grass";
    }
    isToolSuitable(_perm, tool, prefs) {
        if (prefs.coverage.leaves)
            // There is no tags for shears, which is unfortunate but is
            // understandable because the vanilla Minecraft has only one type of shears.
            return tool.typeId === "minecraft:shears" ||
                tool.tags.has("minecraft:is_hoe");
        else
            return false;
    }
}
/// Mixin for azalea-like leaves.
function AzaleaLike(base) {
    class AzaleaLike extends base {
        breakingSoundId() {
            return "dig.azalea_leaves";
        }
        isEquivalentTo(pa, pb) {
            // A special case for mining azalea leaves (flowering or
            // not). It should also mine the other variant as long as they
            // have an identical persistence state.
            if (AZALEA_LEAVES_IDS.has(pa.typeId) && AZALEA_LEAVES_IDS.has(pb.typeId))
                if (pa.states.get("persistent_bit") === pb.states.get("persistent_bit"))
                    return true;
            return super.isEquivalentTo(pa, pb);
        }
    }
    return AzaleaLike;
}
// Oak leaves; they can drop apples.
blockProps.addBlockProps("minecraft:oak_leaves", (_a = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots.set(this, lootOfLeaves(new ItemStack("minecraft:oak_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:oak_sapling")),
                new LootPool()
                    .condition(LootCondition.randomChance(1 / 200, 1 / 180, 1 / 160, 1 / 120, 1 / 40))
                    .entry(new ItemStack("minecraft:apple"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots, "f");
        }
    },
    _loots = new WeakMap(),
    _a));
// Spruce leaves
blockProps.addBlockProps("minecraft:spruce_leaves", (_b = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_1.set(this, lootOfLeaves(new ItemStack("minecraft:spruce_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:spruce_sapling"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_1, "f");
        }
    },
    _loots_1 = new WeakMap(),
    _b));
// Birch leaves
blockProps.addBlockProps("minecraft:birch_leaves", (_c = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_2.set(this, lootOfLeaves(new ItemStack("minecraft:birch_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:birch_sapling"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_2, "f");
        }
    },
    _loots_2 = new WeakMap(),
    _c));
// Jungle leaves; they have a much lower chance of dropping sapplings.
blockProps.addBlockProps("minecraft:jungle_leaves", (_d = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_3.set(this, lootOfLeaves(new ItemStack("minecraft:jungle_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 40, 1 / 36, 1 / 32, 1 / 24, 1 / 10))
                    .entry(new ItemStack("minecraft:jungle_sapling"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_3, "f");
        }
    },
    _loots_3 = new WeakMap(),
    _d));
// Acacia leaves
blockProps.addBlockProps("minecraft:acacia_leaves", (_e = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_4.set(this, lootOfLeaves(new ItemStack("minecraft:acacia_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 40, 1 / 36, 1 / 32, 1 / 24, 1 / 10))
                    .entry(new ItemStack("minecraft:acacia_sapling"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_4, "f");
        }
    },
    _loots_4 = new WeakMap(),
    _e));
// Dark oak leaves; they can drop apples.
blockProps.addBlockProps("minecraft:dark_oak_leaves", (_f = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_5.set(this, lootOfLeaves(new ItemStack("minecraft:dark_oak_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 40, 1 / 36, 1 / 32, 1 / 24, 1 / 10))
                    .entry(new ItemStack("minecraft:dark_oak_sapling")),
                new LootPool()
                    .condition(LootCondition.randomChance(1 / 200, 1 / 180, 1 / 160, 1 / 120, 1 / 40))
                    .entry(new ItemStack("minecraft:apple"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_5, "f");
        }
    },
    _loots_5 = new WeakMap(),
    _f));
// Pale oak leaves
blockProps.addBlockProps("minecraft:pale_oak_leaves", (_g = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_6.set(this, lootOfLeaves(new ItemStack("minecraft:pale_oak_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:pale_oak_sapling"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_6, "f");
        }
    },
    _loots_6 = new WeakMap(),
    _g));
// Mangrove leaves; they don't drop saplings.
blockProps.addBlockProps("minecraft:mangrove_leaves", (_h = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_7.set(this, lootOfLeaves(new ItemStack("minecraft:mangrove_leaves"), []));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_7, "f");
        }
        miningWay(origin, perm, prefs) {
            // A special case for hanging propagules. NOTE: There is
            // similar code in ./tree.ts. Can we somehow unify these?
            switch (perm.typeId) {
                case "minecraft:mangrove_propagule":
                    // Quick mining should only propagate to hanging ones
                    // but ignore their growth stage. This means immature
                    // hanging propagules will be mined as a "bonus" but
                    // will drop nothing.
                    if (perm.states.get("hanging"))
                        return MiningWay.MineAsABonus;
            }
            return super.miningWay(origin, perm, prefs);
        }
    },
    _loots_7 = new WeakMap(),
    _h));
// Cherry leaves
blockProps.addBlockProps("minecraft:cherry_leaves", (_j = class extends LeavesProperties {
        constructor() {
            super(...arguments);
            _loots_8.set(this, lootOfLeaves(new ItemStack("minecraft:cherry_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:cherry_sapling"))
            ]));
        }
        breakingSoundId() {
            return "break.cherry_leaves";
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_8, "f");
        }
    },
    _loots_8 = new WeakMap(),
    _j));
// Azalea leaves
blockProps.addBlockProps("minecraft:azalea_leaves", (_k = class extends AzaleaLike(LeavesProperties) {
        constructor() {
            super(...arguments);
            _loots_9.set(this, lootOfLeaves(new ItemStack("minecraft:azalea_leaves"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:azalea"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_9, "f");
        }
    },
    _loots_9 = new WeakMap(),
    _k));
blockProps.addBlockProps("minecraft:azalea_leaves_flowered", (_l = class extends AzaleaLike(LeavesProperties) {
        constructor() {
            super(...arguments);
            _loots_10.set(this, lootOfLeaves(new ItemStack("minecraft:azalea_leaves_flowered"), [new LootPool()
                    .condition(LootCondition.randomChance(1 / 20, 1 / 16, 1 / 12, 1 / 10))
                    .entry(new ItemStack("minecraft:flowering_azalea"))
            ]));
        }
        lootTable() {
            return __classPrivateFieldGet(this, _loots_10, "f");
        }
    },
    _loots_10 = new WeakMap(),
    _l));
//# sourceMappingURL=leaves.js.map
