import { BlockProperties, blockProps } from "../../block-properties.js";
import { DiamondTier, IronTier, StoneTier } from "../mixins.js";
/// Base class for all mineral blocks.
class MineralBlockProperties extends BlockProperties {
    breakingSoundId() {
        return "dig.stone";
    }
    isToolSuitable(_perm, tool, prefs) {
        if (prefs.coverage.minerals)
            return tool.tags.has("minecraft:is_pickaxe");
        else
            return false;
    }
}
const COPPER_BLOCK_VARIANTS = new Set([
    // Non-waxed
    "minecraft:copper_block",
    "minecraft:exposed_copper",
    "minecraft:weathered_copper",
    "minecraft:oxidized_copper",
    // Waxed
    "minecraft:waxed_copper",
    "minecraft:waxed_exposed_copper",
    "minecraft:waxed_weathered_copper",
    "minecraft:waxed_oxidized_copper",
]);
/// Base class for copper block variants
class CopperBlockProperties extends StoneTier(MineralBlockProperties) {
    isEquivalentTo(pa, pb) {
        // Ignore variance in oxidisation levels and waxing.
        return COPPER_BLOCK_VARIANTS.has(pa.typeId) && COPPER_BLOCK_VARIANTS.has(pb.typeId);
    }
}
// Block of Coal
blockProps.addBlockProps("minecraft:coal_block", MineralBlockProperties);
// Block of Copper and its variants
for (const id of COPPER_BLOCK_VARIANTS) {
    blockProps.addBlockProps(id, CopperBlockProperties);
}
// Block of Diamond
blockProps.addBlockProps("minecraft:diamond_block", IronTier(MineralBlockProperties));
// Block of Emerald
blockProps.addBlockProps("minecraft:emerald_block", IronTier(MineralBlockProperties));
// Block of Gold
blockProps.addBlockProps("minecraft:gold_block", IronTier(MineralBlockProperties));
// Block of Iron
blockProps.addBlockProps("minecraft:iron_block", StoneTier(MineralBlockProperties));
// Block of Lapis Lazuli
blockProps.addBlockProps("minecraft:lapis_block", StoneTier(MineralBlockProperties));
// Block of Netherite
blockProps.addBlockProps("minecraft:netherite_block", DiamondTier(MineralBlockProperties));
// Block of Quartz
blockProps.addBlockProps("minecraft:quartz_block", MineralBlockProperties);
// Block of Redstone
blockProps.addBlockProps("minecraft:redstone_block", MineralBlockProperties);
//# sourceMappingURL=minerals.js.map
