var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _immatureLoots, _matureLoots, _a, _immatureLoots_1, _b, _grassLoots, _fernLoots, _c;
import { ItemStack } from "../../../cicada-lib/dist/item/stack.js";
import { BlockProperties, blockProps } from "../../block-properties.js";
import { LootTable, LootCondition, LootEntry, LootPool } from "../../loot-table.js";
import { DiscreteUniformDrops, IgnoringAllStates, IgnoringState } from "../mixins.js";
/// Base class for all plants.
class PlantProperties extends BlockProperties {
    isToolSuitable(_perm, _tool, prefs) {
        return prefs.coverage.plants;
    }
}
/// Mixin for plants requiring axes to quick-mine.
function MinedWithAxe(base) {
    class MinedWithAxe extends base {
        breakingSoundId() {
            return "dig.wood";
        }
        isToolSuitable(perm, tool, prefs) {
            if (tool.tags.has("minecraft:is_axe"))
                return super.isToolSuitable(perm, tool, prefs);
            else
                return false;
        }
    }
    return MinedWithAxe;
}
/// Mixin for plants requiring hoes to quick-mine. These plants don't
/// consume the tool durability by default.
function MinedWithHoe(base, consumesDurability = false) {
    class MinedWithHoe extends base {
        breakingSoundId() {
            return "dig.grass";
        }
        isToolSuitable(perm, tool, prefs) {
            if (tool.tags.has("minecraft:is_hoe"))
                return super.isToolSuitable(perm, tool, prefs);
            else
                return false;
        }
        consumesDurability() {
            return consumesDurability;
        }
    }
    return MinedWithHoe;
}
/// Mixin for plants requiring shears to quick-mine. They drop itself when mined.
function MinedWithShears(base) {
    class MinedWithShears extends base {
        breakingSoundId() {
            return "dig.grass";
        }
        isToolSuitable(perm, tool, prefs) {
            if (tool.typeId === "minecraft:shears")
                return super.isToolSuitable(perm, tool, prefs);
            else
                return false;
        }
    }
    return MinedWithShears;
}
/// Mixin for plants requiring either hoes or shears to quick-mine. These
/// plants don't consume the tool durability when mined with a hoe.
function MinedWithHoeOrShears(base) {
    class MinedWithHoeOrShears extends base {
        breakingSoundId() {
            return "dig.grass";
        }
        isToolSuitable(perm, tool, prefs) {
            if (tool.typeId === "minecraft:shears" ||
                tool.tags.has("minecraft:is_hoe"))
                return super.isToolSuitable(perm, tool, prefs);
            else
                return false;
        }
        consumesDurability(_perm, tool) {
            if (tool.tags.has("minecraft:is_hoe"))
                return false;
            else
                return true;
        }
    }
    return MinedWithHoeOrShears;
}
/// Mixin for crops. Their seed drops use binomial distribution.
function Crop(base, item, seed, extraPools) {
    const loots = new LootTable()
        .always([new LootPool().entry(item),
        new LootPool()
            .entry(LootEntry
            .item(seed)
            .rolls(3)
            .binomial(0.57)),
        ...(extraPools ?? [])
    ]);
    class Crop extends base {
        isToolSuitable(perm, tool, prefs) {
            if (perm.states.get("growth") === 7)
                // Fully grown.
                return super.isToolSuitable(perm, tool, prefs);
            else
                return false;
        }
        lootTable() {
            return loots;
        }
    }
    return Crop;
}
/// Mixin for glow-lichen-like plants. Multiple of them can be placed in
/// the area of one block, and mining them causes all in the block to drop.
export function GlowLichenLike(base, cond) {
    // Ignore the difference in direction bits.
    class GlowLichenLike extends IgnoringState(base, "multi_face_direction_bits") {
        lootTable(perm) {
            const bits = Number(perm.states.get("multi_face_direction_bits") || 63);
            const amount = ((bits & 0x01) != 0 ? 1 : 0) +
                ((bits & 0x02) != 0 ? 1 : 0) +
                ((bits & 0x04) != 0 ? 1 : 0) +
                ((bits & 0x08) != 0 ? 1 : 0) +
                ((bits & 0x10) != 0 ? 1 : 0) +
                ((bits & 0x20) != 0 ? 1 : 0);
            const stack = perm.getItemStack(amount);
            if (stack)
                if (cond)
                    return new LootTable().when(cond, [new LootPool().entry(stack)]);
                else
                    return new LootTable().always([new LootPool().entry(stack)]);
            else
                return new LootTable(); // Drop nothing
        }
    }
    return GlowLichenLike;
}
/// Mixin for vine-like plants. Multiple of them can be placed in
/// the area of one block, but mining them causes only one in the block to drop.
function VineLike(base) {
    // Ignore the difference in direction bits.
    return IgnoringState(base, "vine_direction_bits");
}
/// Mixin for bush-like plants. They are insta-mined with shears.
function BushLike(base) {
    class BushLike extends MinedWithShears(base) {
        consumesDurability() {
            return false;
        }
    }
    return BushLike;
}
// Cocoa Beans
blockProps.addBlockProps("minecraft:cocoa", (_a = class extends IgnoringState(MinedWithAxe(PlantProperties), "direction") {
        constructor() {
            super(...arguments);
            _immatureLoots.set(this, new LootTable()
                .always(
            // Only one bean.
            [new LootPool().entry(new ItemStack("minecraft:cocoa_beans", 1))
            ]));
            _matureLoots.set(this, new LootTable()
                .always(
            // Always 3 beans regardless of Fortune.
            [new LootPool().entry(new ItemStack("minecraft:cocoa_beans", 3))
            ]));
        }
        lootTable(perm) {
            if ((perm.states.get("age") ?? 2) === 2)
                return __classPrivateFieldGet(this, _matureLoots, "f");
            else
                return __classPrivateFieldGet(this, _immatureLoots, "f");
        }
    },
    _immatureLoots = new WeakMap(),
    _matureLoots = new WeakMap(),
    _a));
// Melon
blockProps.addBlockProps("minecraft:melon_block", DiscreteUniformDrops(MinedWithAxe(PlantProperties), new ItemStack("minecraft:melon_block"), 3, 7, 9, new ItemStack("minecraft:melon_slice")));
// Pumpkin
blockProps.addBlockProps("minecraft:pumpkin", MinedWithAxe(PlantProperties));
blockProps.addBlockProps("minecraft:carved_pumpkin", MinedWithAxe(PlantProperties));
blockProps.addBlockProps("minecraft:lit_pumpkin", MinedWithAxe(PlantProperties));
// Hay bales
blockProps.addBlockProps("minecraft:hay_block", IgnoringAllStates(MinedWithHoe(PlantProperties, true)));
// Nether wart, not sure if it's a plant though.
blockProps.addBlockProps("minecraft:nether_wart", (_b = class extends DiscreteUniformDrops(MinedWithHoe(PlantProperties), new ItemStack("minecraft:nether_wart"), 2, 4, undefined, new ItemStack("minecraft:nether_wart")) {
        constructor() {
            super(...arguments);
            _immatureLoots_1.set(this, new LootTable()
                .always(
            // Only one nether wart.
            [new LootPool().entry(new ItemStack("minecraft:nether_wart"))
            ]));
        }
        breakingSoundId() {
            return "dig.nether_wart";
        }
        lootTable(perm) {
            if ((perm.states.get("age") ?? 3) === 3)
                // Fully grown
                return super.lootTable(perm);
            else
                return __classPrivateFieldGet(this, _immatureLoots_1, "f");
        }
    },
    _immatureLoots_1 = new WeakMap(),
    _b));
// Grass and fern.
// NOTE: Tall grass, tall ferns, and tall flowers are impossible to
// quick-mine and collect loots because breaking their upper halves also
// breaks their lower-halves.
blockProps.addBlockProps("minecraft:tallgrass", (_c = class extends MinedWithHoeOrShears(PlantProperties) {
        constructor() {
            super(...arguments);
            _grassLoots.set(this, new LootTable()
                .when(LootCondition.matchTool().typeId("minecraft:shears"), [new LootPool().entry(new ItemStack("minecraft:tallgrass", { tall_grass_type: "tall" }))])
                .otherwise([new LootPool()
                    .entry(LootEntry
                    .grassLike(new ItemStack("minecraft:wheat_seeds")))
            ]));
            _fernLoots.set(this, new LootTable()
                .when(LootCondition.matchTool().typeId("minecraft:shears"), [new LootPool().entry(new ItemStack("minecraft:tallgrass", { tall_grass_type: "fern" }))])
                .otherwise([new LootPool()
                    .entry(LootEntry
                    .grassLike(new ItemStack("minecraft:wheat_seeds")))
            ]));
        }
        // We would really like to ignore the difference in snow-loggedness
        // in grass and ferns, but snow-logged plants are represented very
        // differently from water-logged blocks. They are actually
        // "minecraft:snow_layer" with the "covered_bit" set, and the plant
        // type is seemingly stored in their NBT, so we can't tell which
        // snow layer is snow-logging what kind of plant.
        lootTable(perm) {
            const grassType = perm.states.get("tall_grass_type");
            switch (grassType) {
                case "default": // Unused
                case "tall":
                    return __classPrivateFieldGet(this, _grassLoots, "f");
                case "fern":
                case "snow": // Unused?
                    return __classPrivateFieldGet(this, _fernLoots, "f");
                case undefined:
                    return super.lootTable(perm);
                default:
                    throw new Error(`Unknown grass type: ${grassType}`);
            }
        }
    },
    _grassLoots = new WeakMap(),
    _fernLoots = new WeakMap(),
    _c));
// Beetroot
blockProps.addBlockProps("minecraft:beetroot", class extends Crop(MinedWithHoe(PlantProperties), new ItemStack("minecraft:beetroot"), new ItemStack("minecraft:beetroot_seeds")) {
    breakingSoundId() {
        return "dig.wood";
    }
});
// Bush and Firefly Bush
blockProps.addBlockProps("minecraft:bush", class extends BushLike(PlantProperties) {
    breakingSoundId() {
        return "dig.grass";
    }
});
blockProps.addBlockProps("minecraft:firefly_bush", class extends BushLike(PlantProperties) {
    breakingSoundId() {
        return "block.sweet_berry_bush.break";
    }
});
// Carrot
blockProps.addBlockProps("minecraft:carrots", Crop(MinedWithHoe(PlantProperties), new ItemStack("minecraft:carrot"), new ItemStack("minecraft:carrot")));
// Glow Lichen
blockProps.addBlockProps("minecraft:glow_lichen", GlowLichenLike(MinedWithShears(PlantProperties)));
// Short and Tall Dry Grass
blockProps.addBlockProps("minecraft:short_dry_grass", class extends BushLike(PlantProperties) {
    breakingSoundId() {
        return "dig.grass";
    }
});
blockProps.addBlockProps("minecraft:tall_dry_grass", class extends BushLike(PlantProperties) {
    breakingSoundId() {
        return "dig.grass";
    }
});
// Potato
blockProps.addBlockProps("minecraft:potatoes", Crop(MinedWithHoe(PlantProperties), new ItemStack("minecraft:potato"), new ItemStack("minecraft:potato"), [new LootPool()
        .condition(LootCondition.randomChance(0.02))
        .entry(new ItemStack("minecraft:poisonous_potato"))
]));
// Vines
blockProps.addBlockProps("minecraft:vine", class extends VineLike(MinedWithShears(PlantProperties)) {
    breakingSoundId() {
        return "dig.roots";
    }
});
// Wheat
blockProps.addBlockProps("minecraft:wheat", Crop(MinedWithHoe(PlantProperties), new ItemStack("minecraft:wheat"), new ItemStack("minecraft:wheat_seeds")));
//# sourceMappingURL=plants.js.map
