var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _LootTable_conds, _LootPool_rolls, _LootPool_cond, _LootPool_ents, _LootEntry_cond, _LootEntry_weight;
import { ItemBag } from "../cicada-lib/dist/item/bag.js";
import { ItemStack } from "../cicada-lib/dist/item/stack.js";
export function randomIntInClosedInterval(lower, upper) {
    return Math.floor(Math.random() * (upper - lower + 1) + lower);
}
/** The scripting API does not offer access to loot table, and this is a
 * miserable attempt to simulate it.
 */
export class LootTable {
    constructor() {
        _LootTable_conds.set(this, void 0);
        __classPrivateFieldSet(this, _LootTable_conds, [], "f");
    }
    when(cond, pools) {
        __classPrivateFieldGet(this, _LootTable_conds, "f").push([cond, pools]);
        return this;
    }
    otherwise(pools) {
        return this.always(pools);
    }
    always(pools) {
        return this.when(LootCondition.always(), pools);
    }
    execute(tool) {
        const items = new ItemBag();
        for (const [cond, pools] of __classPrivateFieldGet(this, _LootTable_conds, "f")) {
            if (cond.evaluate(tool)) {
                for (const pool of pools)
                    items.merge(pool.execute(tool));
                break;
            }
        }
        return items;
    }
}
_LootTable_conds = new WeakMap();
export class LootPool {
    constructor() {
        _LootPool_rolls.set(this, void 0);
        _LootPool_cond.set(this, void 0);
        _LootPool_ents.set(this, void 0);
        __classPrivateFieldSet(this, _LootPool_rolls, { min: 1, max: 1 }, "f");
        __classPrivateFieldSet(this, _LootPool_ents, [], "f");
    }
    rolls(...args) {
        if (args.length == 1) {
            __classPrivateFieldGet(this, _LootPool_rolls, "f").min = args[0];
            __classPrivateFieldGet(this, _LootPool_rolls, "f").max = args[0];
        }
        else {
            __classPrivateFieldGet(this, _LootPool_rolls, "f").min = args[0];
            __classPrivateFieldGet(this, _LootPool_rolls, "f").max = args[1];
        }
        return this;
    }
    condition(cond) {
        __classPrivateFieldSet(this, _LootPool_cond, cond, "f");
        return this;
    }
    entry(arg) {
        if (arg instanceof ItemStack)
            this.entry(LootEntry.item(arg));
        else
            __classPrivateFieldGet(this, _LootPool_ents, "f").push(arg);
        return this;
    }
    execute(tool) {
        const items = new ItemBag();
        const numRolls = randomIntInClosedInterval(__classPrivateFieldGet(this, _LootPool_rolls, "f").min, __classPrivateFieldGet(this, _LootPool_rolls, "f").max);
        for (let i = 0; i < numRolls; i++) {
            if (__classPrivateFieldGet(this, _LootPool_cond, "f") && !__classPrivateFieldGet(this, _LootPool_cond, "f").evaluate(tool))
                continue;
            const totalWeight = __classPrivateFieldGet(this, _LootPool_ents, "f").reduce((acc, ent) => acc + ent.weight(), 0);
            let rand = Math.random() * totalWeight;
            for (const ent of __classPrivateFieldGet(this, _LootPool_ents, "f")) {
                rand -= ent.weight();
                if (rand < 0) {
                    items.merge(ent.execute(tool));
                    break;
                }
            }
        }
        return items;
    }
}
_LootPool_rolls = new WeakMap(), _LootPool_cond = new WeakMap(), _LootPool_ents = new WeakMap();
export class LootCondition {
    static always() {
        return new LootCondition.Always();
    }
    static not(cond) {
        return new LootCondition.Not(cond);
    }
    static and(conds) {
        return new LootCondition.And(conds);
    }
    static or(conds) {
        return new LootCondition.Or(conds);
    }
    static randomChance(noFortune, fortune1, fortune2, fortune3, fortune4plus) {
        return new LootCondition.RandomChance(noFortune, fortune1, fortune2, fortune3, fortune4plus);
    }
    static matchTool() {
        return new LootCondition.MatchTool();
    }
}
(function (LootCondition) {
    var _Not_cond, _And_conds, _Or_conds, _RandomChance_chances, _MatchTool_typeId, _MatchTool_enchantments;
    class Always extends LootCondition {
        evaluate() {
            return true;
        }
    }
    LootCondition.Always = Always;
    class Not extends LootCondition {
        constructor(cond) {
            super();
            _Not_cond.set(this, void 0);
            __classPrivateFieldSet(this, _Not_cond, cond, "f");
        }
        evaluate(tool) {
            return !__classPrivateFieldGet(this, _Not_cond, "f").evaluate(tool);
        }
    }
    _Not_cond = new WeakMap();
    LootCondition.Not = Not;
    class And extends LootCondition {
        constructor(conds) {
            super();
            _And_conds.set(this, void 0);
            __classPrivateFieldSet(this, _And_conds, conds, "f");
        }
        evaluate(tool) {
            for (const cond of __classPrivateFieldGet(this, _And_conds, "f")) {
                if (!cond.evaluate(tool))
                    return false;
            }
            return true;
        }
    }
    _And_conds = new WeakMap();
    LootCondition.And = And;
    class Or extends LootCondition {
        constructor(conds) {
            super();
            _Or_conds.set(this, void 0);
            __classPrivateFieldSet(this, _Or_conds, conds, "f");
        }
        evaluate(tool) {
            for (const cond of __classPrivateFieldGet(this, _Or_conds, "f")) {
                if (cond.evaluate(tool))
                    return true;
            }
            return false;
        }
    }
    _Or_conds = new WeakMap();
    LootCondition.Or = Or;
    class RandomChance extends LootCondition {
        constructor(noFortune, fortune1, fortune2, fortune3, fortune4plus) {
            super();
            _RandomChance_chances.set(this, void 0);
            __classPrivateFieldSet(this, _RandomChance_chances, [
                noFortune,
                fortune1 ?? noFortune,
                fortune2 ?? (fortune1 ?? noFortune),
                fortune3 ?? (fortune2 ?? (fortune1 ?? noFortune)),
                fortune4plus ?? (fortune3 ?? (fortune2 ?? (fortune1 ?? noFortune)))
            ], "f");
        }
        evaluate(tool) {
            if (tool) {
                const fortune = tool.enchantments.get("fortune") ?? 0;
                const chance = __classPrivateFieldGet(this, _RandomChance_chances, "f")[fortune >= 4 ? 4 : fortune];
                return Math.random() < chance;
            }
            else {
                return Math.random() < __classPrivateFieldGet(this, _RandomChance_chances, "f")[0];
            }
        }
    }
    _RandomChance_chances = new WeakMap();
    LootCondition.RandomChance = RandomChance;
    class MatchTool extends LootCondition {
        constructor() {
            super();
            _MatchTool_typeId.set(this, void 0);
            _MatchTool_enchantments.set(this, void 0);
            __classPrivateFieldSet(this, _MatchTool_enchantments, [], "f");
        }
        typeId(typeId) {
            __classPrivateFieldSet(this, _MatchTool_typeId, typeId, "f");
            return this;
        }
        enchantment(id) {
            __classPrivateFieldGet(this, _MatchTool_enchantments, "f").push({
                id
            });
            return this;
        }
        evaluate(tool) {
            if (!tool)
                // Bare hand definitely doesn't match.
                return false;
            if (__classPrivateFieldGet(this, _MatchTool_typeId, "f") !== undefined && __classPrivateFieldGet(this, _MatchTool_typeId, "f") !== tool.typeId)
                return false;
            for (const ench of __classPrivateFieldGet(this, _MatchTool_enchantments, "f")) {
                if (!tool.enchantments.has(ench.id))
                    return false;
            }
            return true;
        }
    }
    _MatchTool_typeId = new WeakMap(), _MatchTool_enchantments = new WeakMap();
    LootCondition.MatchTool = MatchTool;
})(LootCondition || (LootCondition = {}));
export class LootEntry {
    constructor() {
        _LootEntry_cond.set(this, void 0);
        _LootEntry_weight.set(this, void 0);
        __classPrivateFieldSet(this, _LootEntry_weight, 1.0, "f");
    }
    condition(cond) {
        __classPrivateFieldSet(this, _LootEntry_cond, cond, "f");
        return this;
    }
    weight(w) {
        if (w !== undefined) {
            __classPrivateFieldSet(this, _LootEntry_weight, w, "f");
            return this;
        }
        else {
            return __classPrivateFieldGet(this, _LootEntry_weight, "f");
        }
    }
    evalCondition(tool) {
        if (__classPrivateFieldGet(this, _LootEntry_cond, "f"))
            return __classPrivateFieldGet(this, _LootEntry_cond, "f").evaluate(tool);
        else
            return true;
    }
    static item(stack) {
        return new LootEntry.Item(stack);
    }
    /// https://minecraft.wiki/w/Fortune#Grass,_ferns_and_large_flowers%E2%80%8C[Bedrock_Edition_only]
    static grassLike(seed) {
        return new LootEntry.GrassLike(seed);
    }
    static gravelLike(original, decomposed) {
        return new LootEntry.GravelLike(original, decomposed);
    }
    /// https://minecraft.wiki/w/Mushroom_Block
    static shroomLike(shroom) {
        return new LootEntry.ShroomLike(shroom);
    }
}
_LootEntry_cond = new WeakMap(), _LootEntry_weight = new WeakMap();
(function (LootEntry) {
    var _Item_stack, _Item_rolls, _Item_isMultiplicative, _Item_isDiscreteUniform, _Item_binomial, _Item_limit, _GrassLike_seed, _GravelLike_original, _GravelLike_decomposed, _ShroomLike_shroom;
    class Item extends LootEntry {
        constructor(stack) {
            super();
            _Item_stack.set(this, void 0);
            _Item_rolls.set(this, void 0);
            _Item_isMultiplicative.set(this, void 0);
            _Item_isDiscreteUniform.set(this, void 0);
            _Item_binomial.set(this, void 0);
            _Item_limit.set(this, void 0);
            __classPrivateFieldSet(this, _Item_stack, stack, "f");
            __classPrivateFieldSet(this, _Item_rolls, { min: 1, max: 1 }, "f");
            __classPrivateFieldSet(this, _Item_isMultiplicative, false, "f");
            __classPrivateFieldSet(this, _Item_isDiscreteUniform, false, "f");
            __classPrivateFieldSet(this, _Item_binomial, undefined, "f");
            __classPrivateFieldSet(this, _Item_limit, undefined, "f");
        }
        rolls(num) {
            return this.amount(num, num);
        }
        amount(min, max) {
            __classPrivateFieldGet(this, _Item_rolls, "f").min = min;
            __classPrivateFieldGet(this, _Item_rolls, "f").max = max;
            return this;
        }
        multiplicative() {
            __classPrivateFieldSet(this, _Item_isMultiplicative, true, "f");
            return this;
        }
        discreteUniform(limit) {
            __classPrivateFieldSet(this, _Item_isDiscreteUniform, true, "f");
            __classPrivateFieldSet(this, _Item_limit, limit, "f");
            return this;
        }
        /** At least one drop is guaranteed. Calling `.rolls(num)` sets
         * `num + fortune` additional rolls where `fortune` is the
         * fortune level.
         */
        binomial(probability) {
            __classPrivateFieldSet(this, _Item_binomial, probability, "f");
            return this;
        }
        execute(tool) {
            if (!this.evalCondition(tool)) {
                return new ItemBag();
            }
            else if (__classPrivateFieldGet(this, _Item_isMultiplicative, "f")) {
                // https://minecraft.wiki/w/Fortune#Ore
                const fortune = tool?.enchantments.get("fortune") ?? 0;
                const dice = randomIntInClosedInterval(1, fortune + 2);
                const mult = Math.max(1, dice - 1);
                const items = new ItemBag();
                items.add(__classPrivateFieldGet(this, _Item_stack, "f"), mult * randomIntInClosedInterval(__classPrivateFieldGet(this, _Item_rolls, "f").min, __classPrivateFieldGet(this, _Item_rolls, "f").max));
                return items;
            }
            else if (__classPrivateFieldGet(this, _Item_isDiscreteUniform, "f")) {
                const fortune = tool?.enchantments.get("fortune") ?? 0;
                const dice = randomIntInClosedInterval(__classPrivateFieldGet(this, _Item_rolls, "f").min, __classPrivateFieldGet(this, _Item_rolls, "f").max + fortune);
                const items = new ItemBag();
                items.add(__classPrivateFieldGet(this, _Item_stack, "f"), __classPrivateFieldGet(this, _Item_limit, "f") ? Math.min(dice, __classPrivateFieldGet(this, _Item_limit, "f")) : dice);
                return items;
            }
            else if (__classPrivateFieldGet(this, _Item_binomial, "f") !== undefined) {
                const fortune = tool?.enchantments.get("fortune") ?? 0;
                const rolls = randomIntInClosedInterval(__classPrivateFieldGet(this, _Item_rolls, "f").min, __classPrivateFieldGet(this, _Item_rolls, "f").max + fortune);
                const items = new ItemBag();
                let amount = 1;
                for (let i = 0; i < rolls; i++) {
                    if (Math.random() < __classPrivateFieldGet(this, _Item_binomial, "f"))
                        amount++;
                }
                items.add(__classPrivateFieldGet(this, _Item_stack, "f"), amount);
                return items;
            }
            else {
                const items = new ItemBag();
                items.add(__classPrivateFieldGet(this, _Item_stack, "f"), randomIntInClosedInterval(__classPrivateFieldGet(this, _Item_rolls, "f").min, __classPrivateFieldGet(this, _Item_rolls, "f").max));
                return items;
            }
        }
    }
    _Item_stack = new WeakMap(), _Item_rolls = new WeakMap(), _Item_isMultiplicative = new WeakMap(), _Item_isDiscreteUniform = new WeakMap(), _Item_binomial = new WeakMap(), _Item_limit = new WeakMap();
    LootEntry.Item = Item;
    class GrassLike extends LootEntry {
        constructor(seed) {
            super();
            _GrassLike_seed.set(this, void 0);
            __classPrivateFieldSet(this, _GrassLike_seed, seed, "f");
        }
        execute(tool) {
            const dice = randomIntInClosedInterval(1, 8);
            if (dice < 8) {
                // No drops regardless of fortune.
                return new ItemBag();
            }
            else {
                const fortune = tool?.enchantments.get("fortune") ?? 0;
                const maximum = 1 + fortune * 2;
                const items = new ItemBag();
                items.add(__classPrivateFieldGet(this, _GrassLike_seed, "f"), randomIntInClosedInterval(1, maximum));
                return items;
            }
        }
    }
    _GrassLike_seed = new WeakMap();
    LootEntry.GrassLike = GrassLike;
    class GravelLike extends LootEntry {
        constructor(original, decomposed) {
            super();
            _GravelLike_original.set(this, void 0);
            _GravelLike_decomposed.set(this, void 0);
            __classPrivateFieldSet(this, _GravelLike_original, original, "f");
            __classPrivateFieldSet(this, _GravelLike_decomposed, decomposed, "f");
        }
        execute(tool) {
            const silkTouch = tool?.enchantments.has("silk_touch") ?? false;
            const fortune = tool?.enchantments.get("fortune") ?? 0;
            const decompChance = silkTouch ? 0 :
                fortune == 0 ? 0.1 :
                    fortune == 1 ? 0.14 :
                        fortune == 2 ? 0.25 : 1.0;
            const items = new ItemBag();
            items.add(Math.random() < decompChance
                ? __classPrivateFieldGet(this, _GravelLike_decomposed, "f")
                : __classPrivateFieldGet(this, _GravelLike_original, "f"));
            return items;
        }
    }
    _GravelLike_original = new WeakMap(), _GravelLike_decomposed = new WeakMap();
    LootEntry.GravelLike = GravelLike;
    class ShroomLike extends LootEntry {
        constructor(shroom) {
            super();
            _ShroomLike_shroom.set(this, void 0);
            __classPrivateFieldSet(this, _ShroomLike_shroom, shroom, "f");
        }
        execute() {
            const items = new ItemBag();
            const dice = Math.random();
            if (dice < 0.1111) {
                items.add(__classPrivateFieldGet(this, _ShroomLike_shroom, "f"), 1);
            }
            else if (dice < 0.2222) {
                items.add(__classPrivateFieldGet(this, _ShroomLike_shroom, "f"), 2);
            }
            return items;
        }
    }
    _ShroomLike_shroom = new WeakMap();
    LootEntry.ShroomLike = ShroomLike;
})(LootEntry || (LootEntry = {}));
//# sourceMappingURL=loot-table.js.map
