var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _TreePropertyRegistry_treeFor;
import { BlockProperties, MiningWay, blockProps } from "./block-properties.js";
import { IgnoringState } from "./block-properties/mixins.js";
// We consider two wood-like blocks be equivalent as long as their block
// states match, except we ignore their pillar axis.
function WoodLike(tree, soundId, isCovered) {
    const leaves = new Set(tree.leaves ?? []);
    class WoodLikeBlockProperties extends IgnoringState(BlockProperties, "pillar_axis") {
        breakingSoundId() {
            return soundId;
        }
        isToolSuitable(_perm, tool, prefs) {
            if (isCovered(prefs))
                return tool.tags.has("minecraft:is_axe");
            else
                return false;
        }
        miningWay(origin, perm, prefs) {
            if (leaves.has(perm.typeId) && !perm.states.get("persistent_bit")) {
                return MiningWay.MineAsABonus;
            }
            else if (tree.log !== undefined && perm.typeId === tree.log) {
                if (prefs.coverage.logs)
                    return MiningWay.MineRegularly;
            }
            else if (tree.wood !== undefined && perm.typeId === tree.wood) {
                if (prefs.coverage.wood)
                    return MiningWay.MineRegularly;
            }
            else if (tree.strippedLog !== undefined && perm.typeId === tree.strippedLog) {
                if (prefs.coverage.strippedLogs)
                    return MiningWay.MineRegularly;
            }
            else if (tree.strippedWood !== undefined && perm.typeId === tree.strippedWood) {
                if (prefs.coverage.strippedWood)
                    return MiningWay.MineRegularly;
            }
            else if (tree.roots !== undefined && perm.typeId === tree.roots) {
                if (prefs.coverage.logs)
                    return MiningWay.MineRegularly;
            }
            else if (tree.miningWay !== undefined) {
                return tree.miningWay(origin, perm, prefs);
            }
            return MiningWay.LeaveAlone;
        }
    }
    return WoodLikeBlockProperties;
}
export class TreePropertyRegistry {
    constructor() {
        _TreePropertyRegistry_treeFor.set(this, void 0);
        __classPrivateFieldSet(this, _TreePropertyRegistry_treeFor, new Map(), "f");
    }
    /// Register a tree. This also generates and registers block properties
    /// for its `log`, `wood`, `strippedLog`, `strippedWood`, and
    /// `roots`. It does not generate block properties for `leaves`.
    add(tree) {
        // Validation
        for (const part of ["log", "wood", "strippedLog", "strippedWood", "roots"]) {
            // @ts-ignore: TypeScript of course doesn't like this.
            if (tree[part] !== undefined && __classPrivateFieldGet(this, _TreePropertyRegistry_treeFor, "f").has(tree[part]))
                // @ts-ignore: TypeScript of course doesn't like this.
                throw new Error(`${tree[part]} has already been registered as a part of another tree`);
        }
        if (tree.roots !== undefined && tree.rootsSound === undefined)
            throw new Error(`If "roots" is defined "rootsSound" must also be defined`);
        // Generate block properties and register them.
        const woodSound = tree.woodSound ?? "dig.wood";
        if (tree.log !== undefined) {
            blockProps.addBlockProps(tree.log, WoodLike(tree, woodSound, prefs => prefs.coverage.logs));
        }
        if (tree.wood !== undefined) {
            blockProps.addBlockProps(tree.wood, WoodLike(tree, woodSound, prefs => prefs.coverage.wood));
        }
        if (tree.strippedLog !== undefined) {
            blockProps.addBlockProps(tree.strippedLog, WoodLike(tree, woodSound, prefs => prefs.coverage.strippedLogs));
        }
        if (tree.strippedWood !== undefined) {
            blockProps.addBlockProps(tree.strippedWood, WoodLike(tree, woodSound, prefs => prefs.coverage.strippedWood));
        }
        if (tree.roots !== undefined) {
            // Roots count as logs as for coverage.
            blockProps.addBlockProps(tree.roots, WoodLike(tree, tree.rootsSound, prefs => prefs.coverage.logs));
        }
        return this;
    }
}
_TreePropertyRegistry_treeFor = new WeakMap();
// @ts-ignore: Intentionally calling a private constructor.
export const treeProps = new TreePropertyRegistry();
//# sourceMappingURL=tree-properties.js.map
